% [eng result] = resize(img, n, d, ns, verbose)
%
%   [eng result] = resize(img, n, d, ns, verbose) is the energy function of original 
%   image (eng) and the resized original image (result), computed
%   using seam-carving technique. img is the original image (3 channels), n
%   is the new length along d dimension (d=1 -> height resize, d=2 -> width
%   resize. n must be smaller than the original image length. ns is the
%   maximum gap between pixels belonging to the same seam.
%   When verbose parameter is 1 the removed seam are shown over the resized
%   image.
%
%   Luigi Cardamone - cardamoneluigi@hotmail.com
%   Pamela Gotti - pamela.gotti@gmail.com

function [eng eng2 result] = resize(im, n, d, ns, verbose)

%% Checking input parameters
if n>=size(im,1) && d==1
    error('n must be less of %d', size(im,1));
end
if n>=size(im,2) && d==2
        error('n must be less of %d', size(im,2));
end

%% Elaboration of original image
%If the resize involves height, original image has to be transposed, since
%function seam computes vertical seam
if d==1 
    img(:,:,1)=im(:,:,1)';
    img(:,:,2)=im(:,:,2)';
    img(:,:,3)=im(:,:,3)';
    m=size(im,1);
else
    img=im;
    m=size(im,2);
end

%imgtodraw is used to show the seam; it is equal to original image, but
%the seam is showed in black
imgtodraw=img;
   
%Conversion to gray scale image
img=im2double(rgb2gray(uint8(img)));
%% Removing n seam from original image
n=m-n;

for k=1:n
 
    %Computation of vertical seam
    [e v]=seamopt(img, ns);
    %[e v]=energiafinal(img, ns);
    if k==1
        eng=e;
    end
    [h w c]=size(img);
    % showing the seam that is going to be removed
    if verbose==1 
        visSeam(imgtodraw, v, d, k);
    end
    % removing the seam...
    for i=1:h
        if v(i)~=w %if the seam is not at the end of the row, the image pixel are shifted of 1 pixel to the right
            img(i,v(i):w-1)=img(i,(v(i)+1):w);
            imgtodraw(i,v(i):w-1, :)=imgtodraw(i,(v(i)+1):w, :);
        end
    end    
    %resizing the two images, ignoring the last pixel
    imgtodraw=imgtodraw(:,1:end-1,:);
    img=img(:,1:end-1);
end
[eng2 v]=seamopt(img, ns);
% if resizing is on height the final image has to be transposed
if d==1
    imgtodraw2=imgtodraw;
    clear imgtodraw
    imgtodraw(:,:,1)=imgtodraw2(:,:,1)';
    imgtodraw(:,:,2)=imgtodraw2(:,:,2)';
    imgtodraw(:,:,3)=imgtodraw2(:,:,3)';
    eng=eng';
    clear imgtodraw2;
end
result=imgtodraw;



