% [eng vseam] = seamopt(im, k)
%
%     vseam = seamopt (im) is the energy function and the vertical seam 
%     with the minimum amount of energy. vseam is a column vector with the 
%     same number of rows as the input image. 
%     In each row of the vector the horizontal position
%     of the pixels of the vertical seam is contained. im is the grey scale
%     version of original image. k is the maximum gap between pixels
%     belonging to the same seam.
%
%     This version of the function for computing a single seam is more
%     than 10 time faster than the previous version.
%     The optimization involves the computation of M matrix (see seam 
%     carving article). To speed up the computation all the call to the 
%     "min" function had been removed, because each call to "min" function 
%     for a given a vector of 3 pixel function doesn't take in account 
%     previous informations on those pixel.
%
%   Luigi Cardamone - cardamoneluigi@hotmail.com
%   Pamela Gotti - pamela.gotti@gmail.com

function [eng vseam] = seamopt(im, k)

%% Computation of energy function

%Kernels
dx=[-1 0 1];%; -1 0 1; -1 0 1];
dy=dx';

%Here M is used to hold Gx values, just to avoid creating a lot of big
%matrixes

%Partial derivatives
M=conv2(im,dx,'same');
Gy=conv2(im,dy,'same');

%Energy of the picture saved into matrix M
M=abs(M) + abs(Gy);
eng=M;

[h w c]=size(im);

%% Computation of M matrix

globalmin=10000;%very high value
%p1,p2,p3 indicate current pixels
for i=2:h %from second row to the last
    for j=1:w
         if j==1    %first column: initialization of p array
             p=M(i-1, max([1, j-k]):min([j+k, w]));
             minimum=min(p);
             M(i,j)=M(i,j)+minimum;
         else
             if j+k<=w
                 p3=M(i-1,j+k); %taking new element of array p
             end
             %updating p
             if j-k>1
                 p=p(2:end);
             end
             if j+k<=w
                 p(end+1)=p3;
             end
             minimum=min(p);
             M(i,j)=M(i,j)+minimum;            
         end
        if i==h %last row, it's possible to find the min element
            if M(i,j)<globalmin %to use for backtracking
                globalmin=M(i,j);
                posiz=j;
            end
        end
    end
end

%% Looking for the vertical seam
%Vector of seam pixel position
v=zeros(h,1);
v(h)=posiz;

%Backtrack

%Starting from the pixel with the minimum amount of energy in the last row,
%the seam is found through backtracking.
for i=(h-1):-1:1
    p=v(i+1);
    [m v(i)]=min(M(i,max([1, p-k]):min([p+k, w]))); %taking value and position of the i-th row minimum
    if p>k     %if p>=k, then v(i) is not in [p-k p+k] but in [1 2k+1] range; it is necessary to compute the actual value of v(i)
        v(i)=v(i)+p-k-1;
    end
    if v(i)>w   %v(i)>w doesn't correspond to a valid index
        v(i)=w;
    end
end
vseam=v;
